EngineBase

Struct EngineBase 

pub struct EngineBase<D, Q, A>
where D: Driver<Q, A>, Q: RawState,
{ /* private fields */ }
The EngineBase implementation is designed as a type of runtime for executing various Turing machine models, or drivers, according to a specified set of rules encapsulated within a Program<Q, A>.

Implementations

Source

impl<D, Q, A> EngineBase<D, Q, A>
where D: Driver<Q, A>, Q: RawState,

pub fn from_program( program: ProgramBase<Vec<Rule<Q, A>>, Q, A>, ) -> EngineBase<D, Q, A>
where D: Default,

initialize a new instance of the engine using the default driver and given program

pub fn from_driver_with_input<I>(driver: D, input: I) -> EngineBase<D, Q, A>
where I: IntoIterator<Item = A>,

initialize a new engine using the given driver and program

pub const fn from_driver(driver: D) -> EngineBase<D, Q, A>

initialize a new instance of the engine from the given driver

pub fn load( self, program: ProgramBase<Vec<Rule<Q, A>>, Q, A>, ) -> EngineBase<D, Q, A>

load a new program into the engine and return a mutable reference to self

pub fn with_tape<I>(self, input: I) -> EngineBase<D, Q, A>
where I: IntoIterator<Item = A>,

consumes the current instance to create another with the given tape input

pub fn set_program(&mut self, program: ProgramBase<Vec<Rule<Q, A>>, Q, A>)

update the current program for the engine

pub fn with_driver<D2>(self, driver: D2) -> EngineBase<D2, Q, A>
where D2: Driver<Q, A>,

consumes the engine to create another with the given driver

pub fn with_program( self, program: ProgramBase<Vec<Rule<Q, A>>, Q, A>, ) -> EngineBase<D, Q, A>

consumes the current instance to create another with the given program

pub const fn cycles(&self) -> usize

returns a copy of the total number of cycles, or steps, the engine has preformed

pub const fn driver(&self) -> &D

returns a reference to the actor

pub const fn driver_mut(&mut self) -> &mut D

returns a mutable reference to the actor

pub const fn program(&self) -> Option<&ProgramBase<Vec<Rule<Q, A>>, Q, A>>

returns a reference to the program

pub const fn program_mut( &mut self, ) -> Option<&mut ProgramBase<Vec<Rule<Q, A>>, Q, A>>

returns a mutable reference to the program

pub const fn tape(&self) -> &Vec<A>

returns a reference to the output tape

pub const fn tape_mut(&mut self) -> &mut Vec<A>

returns a mutable reference to the output tape

pub fn current_position(&self) -> usize

returns the current position of the driver

pub fn current_state(&self) -> State<&Q>

returns a view of the current state of the driver

pub const fn len(&self) -> usize

returns the length of the output tape

pub const fn is_empty(&self) -> bool

returns true if the output tape is empty

pub const fn has_program(&self) -> bool

returns true if the engine has a program loaded

pub fn extend_tape<I>(&mut self, iter: I)
where I: IntoIterator<Item = A>,

extend the output tape with values from the given iterator

pub fn find_tail<K>(&self, state: State<&Q>, symbol: &A) -> Option<&Tail<Q, A>>
where Q: Eq + Hash, A: Eq + Hash,

returns the tail associated with the head that is equal to the given state and symbol

pub const fn next_cycle(&mut self)

increments the current epoch by a single unit indicating the end of a cycle or step

pub fn reset(&mut self)

reset the engine by clearing the output tape, cycles, and program from the current instance

pub fn is_halted(&self) -> bool
where Q: Halting,

returns true if the driver is in a halted state

pub fn step<Z>(&mut self) -> Result<Z, Error>
where EngineBase<D, Q, A>: TryStep<Output = Z, Error = Error>,

execute a single step of the engine

pub fn run(&mut self) -> Result<(), Error>
where Q: Halting, EngineBase<D, Q, A>: TryStep<Output = Head<Q, A>, Error = Error>,

runs the program until termination (i.e., a halt state is reached, an error occurs, etc.)

pub fn pretty_print(&self) -> String
where A: Debug,

a string representation of the driver’s tape with the current head position highlighted in brackets. 0, 1, 0, [1], 1, 0, 0 for a radius of 3.

pub fn print(&self) -> String
where A: Display,

returns a string representation of the tape with the current head position highlighted in brackets.

pub fn read(&self) -> Result<&A, Error>

read the current symbol at the head of the tape into the internal buffer

pub fn read_head(&self) -> Result<Head<&Q, &A>, Error>

read and return the current head of the machine

pub fn read_uninit(&self) -> Head<&Q, MaybeUninit<&A>>

Reads the current symbol at the head of the tape

Source

impl<Q, A> EngineBase<Head<Q, usize>, Q, A>
where Q: RawState + PartialEq, A: PartialEq,

pub fn tmh( program: ProgramBase<Vec<Rule<Q, A>>, Q, A>, ) -> EngineBase<Head<Q, usize>, Q, A>
where Q: Clone + Default,

initialize a new instance of a turing machine with a moving head using the given program.

pub fn from_state_with_input<I>( state: Q, input: I, ) -> EngineBase<Head<Q, usize>, Q, A>
where I: IntoIterator<Item = A>,

initialize a new instance of the TMH engine from the given state and input