Head

Struct Head 

#[repr(C)]
pub struct Head<Q, A> { pub state: State<Q>, pub symbol: A, }
The Head of a Turing machine is defined to be a two-tuple consisting of a state and a symbol. Our implementation is generic over both the state and symbol types, allowing for flexibility in their representation(s).

Fields

state: State<Q>symbol: A

Implementations

Source

impl<Q, A> Head<Q, A>
where Q: RawState,

The core implementation of the Head providing fundamental methods for its manipulation, including various constructors, accessors, and mutators as well as convienience methods for converting between representations.

pub const fn new(state: Q, symbol: A) -> Head<Q, A>

initialize a new instance of the Head given some state and symbol

pub fn from_state(state: Q) -> Head<Q, A>
where A: Default,

returns a new instance of the head using the given state and a default symbol

pub fn from_symbol(symbol: A) -> Head<Q, A>
where Q: Default,

returns a new instance of the head using the given symbol and a default state

pub fn from_tail(_: Tail<Q, A>) -> Head<Q, A>

create an instance of the Head from a given Tail

pub fn from_tuple(_: (State<Q>, A)) -> Head<Q, A>

Create a new instance from a 2-tuple: (state, symbol)

pub const fn state(&self) -> &State<Q>

returns a reference to the current state

pub const fn state_mut(&mut self) -> &mut State<Q>

returns a mutable reference to the current State

pub const fn symbol(&self) -> &A

returns a reference to the current symbol

pub const fn symbol_mut(&mut self) -> &mut A

returns a mutable reference to the current symbol

pub fn set_state(&mut self, state: Q)

updates the current state

pub fn set_symbol(&mut self, symbol: A)

updates the current symbol

pub fn with_state<Q2>(self, state: Q2) -> Head<Q2, A>

consumes the current instance to create another instance with the given state

pub fn with_symbol<A2>(self, symbol: A2) -> Head<Q, A2>

consumes the current instance to create another instance with the given symbol

pub const fn replace(&mut self, state: State<Q>, symbol: A) -> Head<Q, A>

replaces the current values with the given state and symbol, returning the previous instance of the Head

pub const fn replace_state(&mut self, state: State<Q>) -> State<Q>

replace the current state with the given state, returning the previous state

pub const fn replace_symbol(&mut self, symbol: A) -> A

replace the current symbol with the given symbol, returning the previous symbol

pub const fn swap(&mut self, other: &mut Head<Q, A>)

swap the state and symbol of the current instance with another

pub const fn swap_state(&mut self, other: &mut State<Q>)

swap the current state with another

pub const fn swap_symbol(&mut self, other: &mut A)

swap the current symbol with another

pub fn update(&mut self, state: Option<State<Q>>, symbol: Option<A>)

updates the current State and symbol

pub const fn as_tuple(&self) -> (&State<Q>, &A)

returns a reference to the current state and symbol returing a 2-tuple

pub const fn as_mut_tuple(&mut self) -> (&mut State<Q>, &mut A)

returns a mutable reference to the current state and symbol as a 2-tuple

pub fn into_tuple(self) -> (State<Q>, A)

Consumes the head and returns the current state and symbol as a 2-tuple

pub fn into_rule_with_tail(self, tail: Tail<Q, A>) -> Rule<Q, A>
where Q: RawState,

associates the given tail with the current head, returning a new Rule

pub fn read_tape<T>(self, tape: &[T]) -> Option<&<A as SliceIndex<[T]>>::Output>
where A: SliceIndex<[T]>,

tries reading the given tape using the head as its coordinates.

pub const fn view(&self) -> Head<&Q, &A>

returns a new head with immutable references to the current state and symbol

pub const fn view_mut(&mut self) -> Head<&mut Q, &mut A>

returns a new head with mutable references to the current state and symbol

pub fn step<'a, Q2, B>( &'a mut self, tail: Tail<Q2, B>, ) -> HeadStep<'a, Q, A, Q2, B>
where Q2: RawState,

use the given Tail to initialize a new HeadStep instance for performing a step operation.

Note: this method is lazy, meaning that the actual step is not performed until an actionable method is called on the returned HeadStep instance.

Source

impl<'a, Q, S> Head<&'a Q, &'a S>
where Q: RawState,

pub fn cloned(&self) -> Head<Q, S>
where Q: Clone, S: Clone,

returns a new Head with cloned elements

pub const fn copied(&self) -> Head<Q, S>
where Q: Copy, S: Copy,

returns a new Head with copied elements

Source

impl<'a, Q, S> Head<&'a mut Q, &'a mut S>
where Q: RawState,

pub fn cloned(&self) -> Head<Q, S>
where Q: Clone, S: Clone,

returns a new Head with cloned elements

pub fn copied(&self) -> Head<Q, S>
where Q: Copy, S: Copy,

returns a new Head with copied elements

Source

impl<Q> Head<Q, usize>
where Q: RawState,

pub fn load<A>( self, program: ProgramBase<Vec<Rule<Q, A>>, Q, A>, ) -> EngineBase<Head<Q, usize>, Q, A>
where Q: PartialEq, A: PartialEq,

load the head into an engine loaded with the given program

pub fn shift(self, direction: Direction) -> Head<Q, usize>

pub fn shift_inplace(&mut self, direction: Direction)